;; Servo Motor Control
;;
;; Copyright 2012 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike to mikekohn.net
;;
;; Control 4 server motors over UART (or IR).

.include "msp430x2xx.inc"

; 2.0 ms = 152 interrupts
; 0.3 ms = 23 interrupts
; 0.6 ms = 46 interrupts
; 1.0 ms = 76 interrupts

; 500ms = 38000 interrupts

; HH YAW PTCH THROTTLE YAW_CORRECT
; YAW = 0-126  63
; PITCH = 0-126 63
; THROTTLE = 0-126
; CORR = 0-126 63

RAM equ 0x0200
HEADER equ 152
SHORT equ 23
LONG equ 46

WATCHDOG equ RAM+48

;  r4 =
;  r5 = interupt count (don't think this is needed anymore)
;  r6 = data in from uart
;  r7 = temp in main
;  r8 =
;  r9 = interrupt count
; r10 =
; r11 =
; r12 =
; r13 =
; r14 =
; r15 = temp in interrupt

  .org 0xc000
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; Set up stack pointer
  mov.w #0x0400, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

.if 0
  ;; Set MCLK to 16 MHz external crystal
  bic.w #OSCOFF, SR
  bis.b #XTS, &BCSCTL1
  mov.b #LFXT1S_3, &BCSCTL3
  ;mov.b #LFXT1S_3|XCAP_1, &BCSCTL3
test_osc:
  bic.b #OFIFG, &IFG1
  mov.w #0x00ff, r15
dec_again:
  dec r15
  jnz dec_again
  bit.b #(OFIFG), &IFG1
  jnz test_osc
  mov.b #(SELM_3|SELS), &BCSCTL2
.endif

  ;; Set up output pins
  ;; P2.7 = IR Input
  ;; P1.1 = RX
  ;; P1.2 = TX
  ;; P1.5 = Servo 0
  ;; P2.0 = Servo 1
  ;; P2.1 = Servo 2
  ;; P2.2 = Servo 3
  ;; P2.3 = Debug LED
  mov.b #0x20, &P1DIR
  mov.b #0x20, &P1OUT
  mov.b #0x0f, &P2DIR
  mov.b #0x0f, &P2OUT
  mov.b #0x00, &P2SEL
  mov.b #0x06, &P1SEL
  mov.b #0x06, &P1SEL2

  ;; Set up Timer
  mov.w #210, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  ;; Setup UART
  mov.b #UCSSEL_2|UCSWRST, &UCA0CTL1
  mov.b #0, &UCA0CTL0
  ;mov.b #0x8a, &UCA0BR0
  ;mov.b #0x00, &UCA0BR1
  ;mov.b #0x14, &UCA0BR0
  ;mov.b #0x01, &UCA0BR1
  mov.b #0xf8, &UCA0BR0
  mov.b #0x05, &UCA0BR1
  bic.b #UCSWRST, &UCA0CTL1

  mov.b &speed_table, &RAM+4  ; Servo 0
  mov.b &speed_table, &RAM+5  ; Servo 1
  mov.b &speed_table, &RAM+6  ; Servo 2
  mov.b &speed_table, &RAM+7  ; Servo 3

  mov.w #0, &WATCHDOG
  mov.w #0, r9

  ;; Okay, I can be interrupted now
  eint

main:
  ;bit.b #0x80, &P2IN
  bit.b #UCA0RXIFG, &IFG2
  jz main

  ;; vvvvvmmm   v=value m=motor
  mov.b &UCA0RXBUF, r6
  mov.w #50, &WATCHDOG

  ;; DEBUG
  mov.b r6, &UCA0TXBUF

  ;; r6 will hold motor number
  mov.b r6, r8
  and.b #0x7, r6

  ;; if m >= 4, forget it
  cmp.b #4, r6
  jhs main

  ;; r8 should hold motor value
  rra.w r8
  rra.w r8
  rra.w r8

  ;; if v > 20, forget it
  cmp.b #21,r8
  jhs main

  ;; Do servo position
  ;mov.b &RAM+2, r7
  add.w #speed_table, r8
  ;add.w #RAM+4, r6
  mov.b @r8, (RAM+4)(r6)
  ;mov.b @r8, &RAM+4
  ;mov.b @r8, &RAM+5
  ;mov.b @r8, &RAM+6
  ;mov.b @r8, &RAM+7

  jmp main

timer_interrupt:
  inc.w r5
  inc.w r9
  mov.b &RAM+4, r15
  cmp.w r15, r9
  jl keep_signal_high0
  bis.b #0x20, &P1OUT
keep_signal_high0:
  mov.b &RAM+5, r15
  cmp.w r15, r9
  jl keep_signal_high1
  bis.b #0x01, &P2OUT
keep_signal_high1:
  mov.b &RAM+6, r15
  cmp.w r15, r9
  jl keep_signal_high2
  bis.b #0x02, &P2OUT
keep_signal_high2:
  mov.b &RAM+7, r15
  cmp.w r15, r9
  jl keep_signal_high3
  bis.b #0x04, &P2OUT
keep_signal_high3:

  cmp.w #76*20, r9
  jl not_20_ms_yet
  mov.w #0, r9
  bic.b #0x20, &P1OUT
  bic.b #0x07, &P2OUT
not_20_ms_yet:

.if 0
  dec.w &WATCHDOG
  jnz dont_kick_dog
  ;; If our watchdog reaches 0, stop motor and center tires.
  mov.w #speed_table, r15
  mov.b @r15, &RAM+4  ; Servo 0
  mov.b @r15, &RAM+5  ; Servo 1
  mov.b @r15, &RAM+6  ; Servo 2
  mov.b @r15, &RAM+7  ; Servo 3
dont_kick_dog:
.endif
  reti 

speed_table:
  db  76,  79,  83,  87,  91,  95,  99, 102
  db 106, 110, 114, 118, 122, 126, 129, 133
  db 137, 141, 145, 149, 153
  db 156, 159, 162

  db 76, 76, 77, 77, 78, 79, 79, 80
  db 80, 81, 82, 82, 83, 83, 84, 85 
  db 85, 86, 86, 87, 88, 88, 89, 89
  db 90, 91, 91, 92, 92, 93, 94, 94
  db 95, 96, 96, 97, 97, 98, 99, 99
  db 100, 100, 101, 102, 102, 103, 103, 104
  db 105, 105, 106, 106, 107, 108, 108, 109
  db 109, 110, 111, 111, 112, 112, 113, 114
  db 114, 115, 116, 116, 117, 117, 118, 119
  db 119, 120, 120, 121, 122, 122, 123, 123
  db 124, 125, 125, 126, 126, 127, 128, 128
  db 129, 129, 130, 131, 131, 132, 132, 133
  db 134, 134, 135, 136, 136, 137, 137, 138
  db 139, 139, 140, 140, 141, 142, 142, 143
  db 143, 144, 145, 145, 146, 146, 147, 148
  db 148, 149, 149, 150, 151, 151, 152, 153

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset



